﻿#include <oxstd.h>
#include <oxprob.h>
#include <oxfloat.h>
#import <maximize>
#import <solvenle>
#import <maxsqp>

//Global variables
decl n, X, ksi_hat, sigma_hat; 
decl ksi_Ho= 0.85;   //value of \xi to H_0

// Profile log-likelihood function H_0
floglik_Ho(const vP, const adFunc, const avScore, const amHessian)
{	 	 
     adFunc[0]= -n*log(vP[0])-(1+1/ksi_Ho)*sumc(log(1+(ksi_Ho/vP[0])*X));
	if(avScore)
	{
		avScore[0]= -n/vP[0]+((1+ksi_Ho)/vP[0])*sumc(X ./(vP[0]+ksi_Ho*X));
	}				   
  	if( isnan(adFunc[0]) || isdotinf(adFunc[0]) ) 
 		return 0; 
 	else
 		return 1; 
} 

// Profile log-likelihood function H_1
floglik(const vP, const adFunc, const avScore, const amHessian)
{	  	 
	adFunc[0]= -n*log(vP[1])-(1+1/vP[0])*sumc(log(1+(vP[0]/vP[1])*X));
	if(avScore)
	{
		(avScore[0])[0]= (1/(vP[0]^2))*sumc(log(1+(vP[0]/vP[1])*X))-(1+1/vP[0])*sumc(X ./(vP[1]+vP[0]*X));
		(avScore[0])[1]= -n/vP[1]+((1+vP[0])/vP[1])*sumc(X ./(vP[1]+vP[0]*X));
	}				   
  	if( isnan(adFunc[0]) || isdotinf(adFunc[0]) ) 
 		return 0; 
 	else
 		return 1;
} 

//constrained to maxsqp  
crest(const avF, const vP)
{
  avF[0] = -n/vP[1]+((1+vP[0])/vP[1])*sumc(X ./(vP[1]+vP[0]*X));
  return 1;
}

//constrained to maxsqp under Ho  
crest_Ho(const avF, const vP)
{
   avF[0] = -n/vP[0]+((1+ksi_Ho)/vP[0])*sumc(X ./(vP[0]+ksi_Ho*X));
  return 1;
}

//Cox & Reid profile log-likelihood function	 
floglikPCR(const vP, const adFunc, const avScore, const amHessian)
{
	decl jss, djsss, djssk, tr_mv;

	//CR term
	jss = -n/(vP[1]^2)+((1+vP[0])/(vP[1]^2))*sumc( (X .*(2*vP[1]+vP[0]*X)) ./((vP[1]+vP[0]*X) .^2) );
	tr_mv= 1/(1+ksi_hat);		

	// for the analytical derivates
	djssk = - (1/(vP[0]*vP[1]^2))*sumc(X ./(vP[1]+ vP[0]*X))
		    - (1/(vP[0]*vP[1]))*sumc(X ./((vP[1]+ vP[0]*X) .^2))
			+ (2*(1+vP[0])/vP[0])*sumc(X ./((vP[1]+ vP[0]*X) .^3));				
	djsss = (2*n/(vP[1]^3)) - (2*(1+vP[0])/(vP[1]^3))*sumc((X .*(2*vP[1]+vP[0]*X)) ./((vP[1]+vP[0]*X) .^2))
			- (2*(1+vP[0])/vP[1])*sumc(X ./((vP[1]+ vP[0]*X) .^3));	
                             
	// function to maximize
	adFunc[0]= -n*log(vP[1])-(1+1/vP[0])*sumc(log(1+(vP[0]/vP[1])*X)) - 0.5*log(jss) + (vP[0])*tr_mv;
	if(avScore)
	{					 
		(avScore[0])[0]= (1/(vP[0]^2))*sumc(log(1+(vP[0]/vP[1])*X))-(1+1/vP[0])*sumc(X ./(vP[1]+vP[0]*X))-0.5*djssk/jss + tr_mv;
		(avScore[0])[1]= -n/vP[1]+((1+vP[0])/vP[1])*sumc(X ./(vP[1]+vP[0]*X)) -0.5*djsss/jss;
	}				   
 	
  	if( isnan(adFunc[0]) || isdotinf(adFunc[0]) ) 
 		return 0; 
 	else
 		return 1; 
} 

//Cox & Reid profile log-likelihood function H_0 
floglikPCR_Ho(const vP, const adFunc, const avScore, const amHessian)
{	 
	decl jss, djsss,djssk;
	decl tr_mv;

	// para o termo de CR  
	jss = -n/(vP[0]^2)+((1+ksi_Ho)/(vP[0]^2))*sumc(	(X .*(2*vP[0]+ksi_Ho*X)) ./((vP[0]+ksi_Ho*X) .^2));
	tr_mv= 1/(1+ksi_hat);	  

	// for the analytical derivates
	djssk = -(1/(ksi_Ho*vP[0]^2))*sumc(X ./(ksi_Ho*X+vP[0]))
		    - (1/(ksi_Ho*vP[0]))*sumc(X  ./((ksi_Ho*X+vP[0]) .^2))
			+ (2*(1+ksi_Ho)/ksi_Ho)*sumc(X ./((ksi_Ho*X+vP[0]) .^3));			
	djsss =  (2*n/(vP[0]^3)) - (2*(1+ksi_Ho)/(vP[0]^3))*sumc((X .*(2*vP[0]+ksi_Ho*X)) ./((ksi_Ho*X+vP[0]) .^2))
			 -(2*(1+ksi_Ho)/vP[0])*sumc(X ./((ksi_Ho*X+vP[0]) .^3));  	
                             
	// function to maximize
	adFunc[0]= -n*log(vP[0])-(1+1/ksi_Ho)*sumc(log(1+(ksi_Ho/vP[0])*X)) - 0.5*log(jss) + (ksi_Ho)*tr_mv;
	if(avScore)
	{				
		avScore[0]= -n/vP[0]+((1+ksi_Ho)/vP[0])*sumc(X ./(ksi_Ho*X+vP[0])) - 0.5*djsss/jss;
	}				   
 	
  	if( isnan(adFunc[0]) || isdotinf(adFunc[0]) ) 
 		return 0; 
 	else
 		return 1; 
} 

//Severini profile log-likelihood function 
floglikPBNS(const vP, const adFunc, const avScore, const amHessian)
{
	decl jss, djsss, djssk;
	decl Iv, dIvk, dIvs;

	// para o termo de BNS  
	jss = -n/(vP[1]^2)+((1+vP[0])/(vP[1]^2))*sumc((X .*(2*vP[1]+vP[0]*X)) ./((vP[1]+vP[0]*X) .^2));
	Iv= sumc(	( -1/vP[1]+((1+vP[0])/vP[1])*(X ./(vP[1]+vP[0]*X)) ) .*
				( -1/sigma_hat+((1+ksi_hat)/sigma_hat)*(X ./(sigma_hat+ksi_hat*X)) )  );
	
	// for the analytical derivates
	djssk = - (1/(vP[0]*vP[1]^2))*sumc(X ./(vP[1]+ vP[0]*X))
		    - (1/(vP[0]*vP[1]))*sumc(X ./((vP[1]+ vP[0]*X) .^2))
			+ (2*(1+vP[0])/vP[0])*sumc(X ./((vP[1]+ vP[0]*X) .^3));			
	djsss = (2*n/(vP[1]^3)) - (2*(1+vP[0])/(vP[1]^3))*sumc((X .*(2*vP[1]+vP[0]*X)) ./((vP[1]+vP[0]*X) .^2))
			- (2*(1+vP[0])/vP[1])*sumc(X ./((vP[1]+ vP[0]*X) .^3));	 
	dIvk=sumc(	( ((1+vP[0])/vP[0])*(X ./((vP[1]+vP[0]*X) .^2)) - (1/(vP[1]*vP[0]))*(X ./(vP[1]+vP[0]*X)) ) .*
				( -1/sigma_hat+((1+ksi_hat)/sigma_hat)*(X ./(sigma_hat+ksi_hat*X)) )   );
			
	dIvs=sumc(	( 1/(vP[1]^2)-((1+vP[0])/(vP[1]^2))*((X .*(2*vP[1]+vP[0]*X)) ./((vP[1]+vP[0]*X) .^2)) ) .*
				( -1/sigma_hat+((1+ksi_hat)/sigma_hat)*(X ./(sigma_hat+ksi_hat*X)) )   );		
                             
	// function to maximize
	adFunc[0]= -n*log(vP[1])-(1+1/vP[0])*sumc(log(1+(vP[0]/vP[1])*X)) + 0.5*log(jss) - log(Iv);
	if(avScore)
	{					 
		(avScore[0])[0]= (1/(vP[0]^2))*sumc(log(1+(vP[0]/vP[1])*X))-(1+1/vP[0])*sumc(X ./(vP[1]+vP[0]*X)) + 0.5*djssk/jss -dIvk/Iv;
		(avScore[0])[1]= -n/vP[1]+((1+vP[0])/vP[1])*sumc(X ./(vP[1]+vP[0]*X)) + 0.5*djsss/jss - dIvs/Iv;
	}				   
 	
  	if( isnan(adFunc[0]) || isdotinf(adFunc[0]) ) 
 		return 0; 
 	else
 		return 1; 
} 

//Severini profile log-likelihood function H_0 
floglikPBNS_Ho(const vP, const adFunc, const avScore, const amHessian)
{
	decl jss, djsss, djssk;
	decl Iv, dIvk, dIvs;

	// para o termo de BNS  
	jss = -n/(vP[0]^2)+((1+ksi_Ho)/(vP[0]^2))*sumc((X .*(2*vP[0]+ksi_Ho*X)) ./((vP[0]+ksi_Ho*X) .^2));
	Iv= sumc(	( -1/vP[0]+((1+ksi_Ho)/vP[0])*(X ./(vP[0]+ksi_Ho*X)) ) .*
				( -1/sigma_hat+((1+ksi_hat)/sigma_hat)*(X ./(sigma_hat+ksi_hat*X)) )  );

	// for the analytical derivates		
	djsss = (2*n/(vP[0]^3)) - (2*(1+ksi_Ho)/(vP[0]^3))*sumc((X .*(2*vP[0]+ksi_Ho*X)) ./((vP[0]+ksi_Ho*X) .^2))
			- (2*(1+ksi_Ho)/vP[0])*sumc(X ./((vP[0]+ ksi_Ho*X) .^3));	
	dIvs=sumc(
				( 1/(vP[0]^2)-((1+ksi_Ho)/(vP[0]^2))*((X .*(2*vP[0]+ksi_Ho*X)) ./((vP[0]+ksi_Ho*X) .^2)) ) .*
				( -1/sigma_hat+((1+ksi_hat)/sigma_hat)*(X ./(sigma_hat+ksi_hat*X)) )
			  );  	
                             
	// function to maximize
	adFunc[0]= -n*log(vP[0])-(1+1/ksi_Ho)*sumc(log(1+(ksi_Ho/vP[0])*X)) + 0.5*log(jss) - log(Iv);
	if(avScore)
	{					 
		avScore[0]= -n/vP[0]+((1+ksi_Ho)/vP[0])*sumc(X ./(vP[0]+ksi_Ho*X)) + 0.5*djsss/jss - dIvs/Iv;
	}				   
 	
  	if( isnan(adFunc[0]) || isdotinf(adFunc[0]) ) 
 		return 0; 
 	else
 		return 1; 
}

//Fraser e Reid profile log-likelihood function 
floglikPBNFR(const vP, const adFunc, const avScore, const amHessian)
{
	decl lp, jss, djsss, djssk;
	decl Iv, dIvk, dIvs;
	decl V, lsx;


	lp=-n*log(vP[1])-(1+1/vP[0])*sumc(log(1+(vP[0]/vP[1])*X));	
	jss = -n/(vP[1]^2)+((1+vP[0])/(vP[1]^2))*sumc((X .*(2*vP[1]+vP[0]*X)) ./((vP[1]+vP[0]*X) .^2));
	V= (1/sigma_hat)*X;
	lsx=(1+vP[0]) ./((vP[1]+vP[0]*X) .^2) ;
	Iv= sumc((lsx) .*(V));	

	// for the analytical derivates
	djssk = - (1/(vP[0]*vP[1]^2))*sumc(X ./(vP[1]+ vP[0]*X))
		    - (1/(vP[0]*vP[1]))*sumc(X ./((vP[1]+ vP[0]*X) .^2))
			+ (2*(1+vP[0])/vP[0])*sumc(X ./((vP[1]+ vP[0]*X) .^3));			
	djsss = (2*n/(vP[1]^3)) - (2*(1+vP[0])/(vP[1]^3))*sumc((X .*(2*vP[1]+vP[0]*X)) ./((vP[1]+vP[0]*X) .^2))
			- (2*(1+vP[0])/vP[1])*sumc(X ./((vP[1]+ vP[0]*X) .^3));
	dIvk = sumc( ((vP[1]-(2+vP[0]).*X) ./((vP[1]+vP[0]*X) .^3) ) .*( V ));			
	dIvs=sumc((-2*(1+vP[0]) ./((vP[1]+vP[0]*X) .^3) ) .*( V));
                             
	// function to maximize
	adFunc[0]= lp + 0.5*log(jss) - log(Iv);
	if(avScore)
	{					 
		(avScore[0])[0]= (1/(vP[0]^2))*sumc(log(1+(vP[0]/vP[1])*X))-(1+1/vP[0])*sumc(X ./(vP[1]+vP[0]*X)) + 0.5*djssk/jss -dIvk/Iv;
		(avScore[0])[1]= -n/vP[1]+((1+vP[0])/vP[1])*sumc(X ./(vP[1]+vP[0]*X)) + 0.5*djsss/jss - dIvs/Iv;
	}				   
 	
  	if( isnan(adFunc[0]) || isdotinf(adFunc[0]) ) 
 		return 0; 
 	else
 		return 1; 
}

//Fraser e Reid profile log-likelihood function H_0 	 
floglikPBNFR_Ho(const vP, const adFunc, const avScore, const amHessian)
{
	decl lp, jss, djsss, djssk;
	decl Iv, dIvk, dIvs;
	decl V, lsx;

	//lp de BNFR	
	lp=-n*log(vP[0])-(1+1/ksi_Ho)*sumc(log(1+(ksi_Ho/vP[0])*X));	
	jss = -n/(vP[0]^2)+((1+ksi_Ho)/(vP[0]^2))*sumc((X .*(2*vP[0]+ksi_Ho*X)) ./((vP[0]+ksi_Ho*X) .^2));
	V= (1/sigma_hat)*X;
	lsx=(1+ksi_Ho) ./((vP[0]+ksi_Ho*X) .^2) ;
	Iv= sumc((lsx) .*(V));	

	// for the analytical derivates
	djssk = - (1/(ksi_Ho*vP[0]^2))*sumc(X ./(vP[0]+ ksi_Ho*X))
		    - (1/(ksi_Ho*vP[0]))*sumc(X ./((vP[0]+ ksi_Ho*X) .^2))
			+ (2*(1+ksi_Ho)/ksi_Ho)*sumc(X ./((vP[0]+ ksi_Ho*X) .^3));			
	djsss = (2*n/(vP[0]^3)) - (2*(1+ksi_Ho)/(vP[0]^3))*sumc((X .*(2*vP[0]+ksi_Ho*X)) ./((vP[0]+ksi_Ho*X) .^2))
			- (2*(1+ksi_Ho)/vP[0])*sumc(X ./((vP[0]+ ksi_Ho*X) .^3));
	dIvk = sumc( ((vP[0]-(2+ksi_Ho).*X) ./((vP[0]+ksi_Ho*X) .^3) ) .*( V ));			
	dIvs=sumc((-2*(1+ksi_Ho) ./((vP[0]+ksi_Ho*X) .^3) ) .*( V));
                             
	// function to maximize
	adFunc[0]= lp + 0.5*log(jss) - log(Iv);
	if(avScore)
	{					 
		avScore[0]= -n/vP[0]+((1+ksi_Ho)/vP[0])*sumc(X ./(vP[0]+ksi_Ho*X)) + 0.5*djsss/jss - dIvs/Iv;
	}				   
 	
  	if( isnan(adFunc[0]) || isdotinf(adFunc[0]) ) 
 		return 0; 
 	else
 		return 1; 
} 

//Função que calcula a taxa de rejeição dos testes 
taxa_rejeicao(const vetor)
{
	return 1-probchi(vetor, 1);
}

/*********************************************************************** 
/*********************************************************************** 
***********************************************************************/
***********************************************************************/

main() 
{

	//local variables
	decl tempo, arq, Pcrit;
	decl ir1, vp, dfunc_H1, ir0, vp_Ho, dfunc_Ho;
	decl ir1p, vpp, dfunc_H1p, ir0p, vp_Hop, dfunc_Hop, EM_hatp;
	decl ir1cr, vpcr, dfunc_H1cr, ir0cr, vp_Hocr, dfunc_Hocr, EM_hatcr;
	decl ir1bns, vpbns, dfunc_H1bns, ir0bns, vp_Hobns, dfunc_Hobns, EM_hatbns;
	decl ir1bnfr, vpbnfr, dfunc_H1bnfr, ir0bnfr, vp_Hobnfr, dfunc_Hobnfr, EM_hatbnfr;
	decl EM, pcRVB_10, pcRVB_5, pcRVB_1, pcRVB_05,RV_boot;
	decl w_hat,  w_hatp, w_hatcr, w_hatbns, w_hatbnfr;
	decl RV, resultados1, RVp, resultados1p, RVcr, resultados1cr;
	decl RVbns, resultados1bns, RVbnfr, resultados1bnfr;


	tempo=timer();
	
	//creating the file	to save the results
    arq = fopen("dados_reais_2.txt", "w");


	//First data to illustration  	
//	//flood Nidd  (hosking 1986)
//	decl X1, i, t;
//	t=80;  //
//	X1=<97.24;189.02;68.84;72.21;77.26;72.34;91.8;82.54;115.52;78.17;70.14;78.55;79.33;119.28;
//	162.99;74.93;69.49;102.92;83.56;80.75;143.06;69.62;76.61;91.8;83.69;153.04;77.13;67.02;
//	74.15;65.08;76.22;149.3;81.4;94.08;72.34;116.77;97.87;72.34;81.4;131.82;77.52;107.97;78.81;
//	104.19;76.22;91.55;86.48;68.45;67.15;80.11;261.82;68.45;110.48;74.93;87.75;181.59;76.22;
//	104.19;89.02;77.52;74.93;90.28;81.4;95.47;99.14;158.01;76.22;92.82;67.15;81.4;172.92;97.87;
//	179.12;88.76;74.02;85.85;67.02;70.01;76.74;65.6;77.26;213.7;78.81;86.73;70.52;69.49;78.29;
//	81.4;79.33;83.94;111.74;81.65;67.67;75.06;96.86;67.28;78.29;75.97;100.4;89.65;78.04;70.78;
//	81.27;77.78;76.61;92.82;69.49;74.8;72.34;99.14;74.54;104.19;74.54;86.48;151.79;79.46;66.53;
//	88.89;73.61;111.54;78.98;87.76;148.63;87.76;90.88;87.76;80.12;96.25;251.96;75.87;121.73;78.98;
//	107.58;108.14;73.61;88.04;76.72;99.08;131.92;138.72;75.02;133.06;70.49;257.62;99.93;123.71;
//	87.19;157.12;305.75;77;71.91;89.46;226.48;110.98>;
//	for (i=0;i<rows(X1);i++)
//		if (X1[i] >= t)
//			X |= X1[i]- t; 			
//	X = X[1:(rows(X)-1)];


	//Second data to illustration 	
	// Dowjones data (already $37$ exceedances, with the data transformed and using t=2	as threshold 
	X=<0.000709184572180277;0.0747638559468484;0.569914215974656;0.60486679791061;0.00613068921686732;
	0.0023686368038085;1.32060801174698;0.238414883903062;2.60084128499556;1.0738431822197;
	0.399422016348041;0.513888854527835;1.75353991111425;2.86053488509892;0.335574762917147;
	1.2050618935175;0.144607679295341;2.06469963383138;0.317165131631072;0.479741149633519;
	0.379336444515801;0.00807469336581335;0.252084448877767;0.0436835588542479;0.798425429079643;
	0.0699032375338433;0.146270220036683;0.166287795336608;0.213819004692574;0.36480997205426;
	1.21128511171178;2.80951950084858;0.104158822148605;0.302963163061555;0.709722941200887;
	0.649965076358107;0.188659923066893>;

	println(n=rows(X));

	decl xbar, s,sigma_mom,ksi_mom;
	xbar= meanc(X);
	s=moments(X)[2];
	println("s_m",sigma_mom = 0.5*xbar*(((xbar^2)/s^2)+1));
	println("k_m",ksi_mom = -0.5*(((xbar^2)/s^2)-1));   	
	
	// Maximizing the function floglik por MaxSQP (profile)
	vpp= ksi_mom|sigma_mom;
	ir1p = MaxSQP(floglik, &vpp, &dfunc_H1p, 0, FALSE, 0, crest, <-.Inf;0>, <>);
	if (ir1p!=MAX_CONV && ir1p!=MAX_WEAK_CONV){
	println("not converge MaxSQP");
	}
	ksi_hat=vpp[0];
	sigma_hat=vpp[1];
	println("SQP",vpp);
	vp_Hop= sigma_mom;
	ir0p=MaxSQP(floglik_Ho, &vp_Hop, &dfunc_Hop, 0, FALSE, 0, crest_Ho, <0>, <>);
	if (ir0p!=MAX_CONV && ir0p!=MAX_WEAK_CONV){
	println("not converge MaxSQP Ho"); 
	}
	
	// Maximizing the function floglikPCR profile Cox & Reid
	vpcr=ksi_mom|sigma_mom;
	ir1cr = MaxSQP(floglikPCR, &vpcr, &dfunc_H1cr, 0, FALSE, 0, crest, <-.Inf;0>, <>);
	if (ir1cr!=MAX_CONV && ir1cr!=MAX_WEAK_CONV){
	println("not converge CR");
	}
	println("CR",vpcr);
	vp_Hocr= sigma_mom;
	ir0cr=MaxSQP(floglikPCR_Ho, &vp_Hocr, &dfunc_Hocr, 0, FALSE, 0, crest_Ho, <0>, <>);
	if (ir0cr!=MAX_CONV && ir0cr!=MAX_WEAK_CONV){
	println("not converge CR Ho"); 
	}

	// Maximizing the function floglikPBNS profile BN Severini
	vpbns=ksi_mom|sigma_mom;
	ir1bns = MaxSQP(floglikPBNS, &vpbns, &dfunc_H1bns, 0, FALSE, crest, 0, <-.Inf;0>, <>);
	if (ir1bns!=MAX_CONV && ir1bns!=MAX_WEAK_CONV){
	println("not converge Severini"); 
	}
	println("Sev",vpbns);
	vp_Hobns= sigma_mom;
	ir0bns=MaxSQP(floglikPBNS_Ho, &vp_Hobns, &dfunc_Hobns, 0, FALSE, crest_Ho, 0, <0>, <>);
	if (ir0bns!=MAX_CONV && ir0bns!=MAX_WEAK_CONV){
	println("not converge Severini Ho");
	}

	// Maximizing the function floglikPBNFR profile Fraser & Reid
	vpbnfr=ksi_mom|sigma_mom;
	ir1bnfr = MaxSQP(floglikPBNFR, &vpbnfr, &dfunc_H1bnfr, 0, FALSE, crest, 0, <-.Inf;0>, <>);
	if (ir1bnfr!=MAX_CONV && ir1bnfr!=MAX_WEAK_CONV){
	println("not converge FR");
	}
	println("FR",vpbnfr);
	vp_Hobnfr= sigma_mom;
	ir0bnfr=MaxSQP(floglikPBNFR_Ho, &vp_Hobnfr, &dfunc_Hobnfr, 0, FALSE, crest_Ho, 0, <0>, <>);
	if (ir0bnfr!=MAX_CONV && ir0bnfr!=MAX_WEAK_CONV){
	println("not converge FR Ho");
	}

	//Point estimates
	w_hatp= vpp[0];
	w_hatcr= vpcr[0];
	w_hatbns= vpbns[0];
	w_hatbnfr= vpbnfr[0];
	EM= w_hatp|w_hatcr|w_hatbns|w_hatbnfr;
	
	//likelihood ratio test
	RVp= 2*(dfunc_H1p-dfunc_Hop);
	RVcr= 2*(dfunc_H1cr-dfunc_Hocr);
	RVbns= 2*(dfunc_H1bns-dfunc_Hobns);
	RVbnfr= 2*(dfunc_H1bnfr-dfunc_Hobnfr);	
	//p-value
	resultados1p = taxa_rejeicao(RVp);
	resultados1cr = taxa_rejeicao(RVcr);
	resultados1bns = taxa_rejeicao(RVbns);
	resultados1bnfr = taxa_rejeicao(RVbnfr);

	//printing
    fprintln(arq, "GPD(ksi  , sigma)");
	fprintln(arq, "NUM. OBS.: ", n);
	//point estimation results
	fprintln(arq,"\nPoint estimates","%c",{"EMV"} ,"%r",{"EMVP","EMVcr","EMVbns","EMVbnfr"},"%10.4f",EM);
	//test results
	fprintln(arq,"\nhypothesis testing","\nHo: Ksi= ","%1.2f",ksi_Ho,"%c",{"RVp", "RVcr", "RVbns", "RVbnfr"},
	         "%r",{"test statistics"},"%10.3f",RVp~RVcr~RVbns~RVbnfr);
	fprintln(arq,"\nhypothesis testing","\nHo: Ksi= ","%1.2f",ksi_Ho,"%c",{"RVp", "RVcr", "RVbns", "RVbnfr"},
	         "%r",{"p-value"},"%10.3f",resultados1p~resultados1cr~resultados1bns~resultados1bnfr);

	
}	// end

